@@ -35,6 +35,9 @@ static void
 cb_rectangle_toggled               (GtkToggleButton    *tb,
                                     ScreenshotData     *sd);
 static void
+cb_radiobutton_activate            (GtkToggleButton    *tb,
+                                    GtkWidget          *widget);
+static void
 cb_show_mouse_toggled              (GtkToggleButton    *tb,
                                     ScreenshotData     *sd);
 static void
@@ -44,9 +47,6 @@ static void
 cb_toggle_set_sensi                (GtkToggleButton    *tb,
                                     GtkWidget          *widget);
 static void
-cb_toggle_set_insensi              (GtkToggleButton    *tb,
-                                    GtkWidget          *widget);
-static void
 cb_open_toggled                    (GtkToggleButton    *tb,
                                     ScreenshotData     *sd);
 static void
@@ -127,6 +127,14 @@ static void cb_rectangle_toggled (GtkToggleButton *tb, ScreenshotData *sd)
 
 
 
+/* Confirm the selected screenshot options and proceed to capture */
+static void cb_radiobutton_activate (GtkToggleButton *tb, GtkWidget *widget)
+{
+  gtk_dialog_response (GTK_DIALOG (widget), GTK_RESPONSE_OK);
+}
+
+
+
 /* Set whether the mouse should be captured when the button is toggled */
 static void cb_show_mouse_toggled (GtkToggleButton *tb, ScreenshotData   *sd)
 {
@@ -156,15 +164,6 @@ cb_toggle_set_sensi (GtkToggleButton *tb, GtkWidget *widget)
 
 
 
-/* Set the widget active if the toggle button is inactive */
-static void
-cb_toggle_set_insensi (GtkToggleButton *tb, GtkWidget *widget)
-{
-  gtk_widget_set_sensitive (widget, !gtk_toggle_button_get_active (tb));
-}
-
-
-
 static void cb_open_toggled (GtkToggleButton *tb, ScreenshotData *sd)
 {
   if (gtk_toggle_button_get_active (tb))
@@ -710,7 +709,7 @@ GtkWidget *screenshooter_region_dialog_new (ScreenshotData *sd, gboolean plugin)
   gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
   gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
   gtk_container_set_border_width (GTK_CONTAINER (dlg), 0);
-  gtk_window_set_icon_name (GTK_WINDOW (dlg), "applets-screenshooter");
+  gtk_window_set_icon_name (GTK_WINDOW (dlg), "org.xfce.screenshooter");
   gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
 
   /* Create the main alignment for the dialog */
@@ -779,6 +778,9 @@ GtkWidget *screenshooter_region_dialog_new (ScreenshotData *sd, gboolean plugin)
   g_signal_connect (G_OBJECT (fullscreen_button), "toggled",
                     G_CALLBACK (cb_fullscreen_screen_toggled),
                     sd);
+  g_signal_connect (G_OBJECT (fullscreen_button), "activate",
+                    G_CALLBACK (cb_radiobutton_activate),
+                    dlg);
 
   /* Active window */
   active_window_button =
@@ -794,6 +796,9 @@ GtkWidget *screenshooter_region_dialog_new (ScreenshotData *sd, gboolean plugin)
   g_signal_connect (G_OBJECT (active_window_button), "toggled",
                     G_CALLBACK (cb_active_window_toggled),
                     sd);
+  g_signal_connect (G_OBJECT (active_window_button), "activate",
+                    G_CALLBACK (cb_radiobutton_activate),
+                    dlg);
 
   /* Rectangle */
   rectangle_button =
@@ -806,17 +811,20 @@ GtkWidget *screenshooter_region_dialog_new (ScreenshotData *sd, gboolean plugin)
                                _("Select a region to be captured by clicking a point of "
                                  "the screen without releasing the mouse button, "
                                  "dragging your mouse to the other corner of the region, "
-                                 "and releasing the mouse button."));
+                                 "and releasing the mouse button.\n\n"
+                                 "Press Ctrl while dragging to move the region."));
 
   g_signal_connect (G_OBJECT (rectangle_button), "toggled",
                     G_CALLBACK (cb_rectangle_toggled), sd);
+  g_signal_connect (G_OBJECT (rectangle_button), "activate",
+                    G_CALLBACK (cb_radiobutton_activate),
+                    dlg);
 
   /* Create show mouse checkbox */
   show_mouse_checkbox =
     gtk_check_button_new_with_label (_("Capture the mouse pointer"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (show_mouse_checkbox),
                                 (sd->show_mouse == 1));
-  gtk_widget_set_sensitive (show_mouse_checkbox, (sd->region != SELECT));
   gtk_widget_set_tooltip_text (show_mouse_checkbox,
                                _("Display the mouse pointer on the screenshot"));
   gtk_box_pack_start (GTK_BOX (area_box),
@@ -824,8 +832,6 @@ GtkWidget *screenshooter_region_dialog_new (ScreenshotData *sd, gboolean plugin)
                       FALSE, 5);
   g_signal_connect (G_OBJECT (show_mouse_checkbox), "toggled",
                     G_CALLBACK (cb_show_mouse_toggled), sd);
-  g_signal_connect (G_OBJECT (rectangle_button), "toggled",
-                    G_CALLBACK (cb_toggle_set_insensi), show_mouse_checkbox);
 
   /* Create the main box for the delay stuff */
   delay_main_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
@@ -937,7 +943,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
   gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
   gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
   gtk_container_set_border_width (GTK_CONTAINER (dlg), 0);
-  gtk_window_set_icon_name (GTK_WINDOW (dlg), "applets-screenshooter");
+  gtk_window_set_icon_name (GTK_WINDOW (dlg), "org.xfce.screenshooter");
   gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
 
   /* Create the main alignment for the dialog */
@@ -997,6 +1003,8 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
                                 (sd->action & SAVE));
   g_signal_connect (G_OBJECT (save_radio_button), "toggled",
                     G_CALLBACK (cb_save_toggled), sd);
+  g_signal_connect (G_OBJECT (save_radio_button), "activate",
+                    G_CALLBACK (cb_radiobutton_activate), dlg);
   gtk_widget_set_tooltip_text (save_radio_button, _("Save the screenshot to a PNG file"));
   gtk_grid_attach (GTK_GRID (actions_grid), save_radio_button, 0, 0, 1, 1);
 
@@ -1014,6 +1022,8 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
                                     (sd->action & CLIPBOARD));
       g_signal_connect (G_OBJECT (clipboard_radio_button), "toggled",
                         G_CALLBACK (cb_clipboard_toggled), sd);
+      g_signal_connect (G_OBJECT (clipboard_radio_button), "activate",
+                        G_CALLBACK (cb_radiobutton_activate), dlg);
       gtk_grid_attach (GTK_GRID (actions_grid), clipboard_radio_button, 0, 1, 1, 1);
     }
 
@@ -1025,6 +1035,8 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
                                 (sd->action & OPEN));
   g_signal_connect (G_OBJECT (open_with_radio_button), "toggled",
                     G_CALLBACK (cb_open_toggled), sd);
+  g_signal_connect (G_OBJECT (open_with_radio_button), "activate",
+                    G_CALLBACK (cb_radiobutton_activate), dlg);
   gtk_widget_set_tooltip_text (open_with_radio_button,
                                _("Open the screenshot with the chosen application"));
   gtk_grid_attach (GTK_GRID (actions_grid), open_with_radio_button, 0, 2, 1, 1);
@@ -1066,6 +1078,8 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
                                  "image hosting service"));
   g_signal_connect (G_OBJECT (imgur_radio_button), "toggled",
                     G_CALLBACK (cb_imgur_toggled), sd);
+  g_signal_connect (G_OBJECT (imgur_radio_button), "activate",
+                    G_CALLBACK (cb_radiobutton_activate), dlg);
   gtk_container_add (GTK_CONTAINER (hbox), imgur_radio_button);
 
   /* Upload to imgur warning info */
@@ -1170,7 +1184,7 @@ gchar
                                    GTK_RESPONSE_ACCEPT,
                                    NULL);
 
-    gtk_window_set_icon_name (GTK_WINDOW (chooser), "applets-screenshooter");
+    gtk_window_set_icon_name (GTK_WINDOW (chooser), "org.xfce.screenshooter");
     gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (chooser),
                                                     TRUE);
     gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (chooser), FALSE);
