@@ -30,7 +30,7 @@
 #include "screenshooter-utils.h"
 #include "screenshooter-simple-job.h"
 
-void screenshooter_upload_to_imgur (const gchar  *image_path,
-                                    const gchar  *title);
+gboolean screenshooter_upload_to_imgur (const gchar  *image_path,
+                                        const gchar  *title);
 
 #endif
